#ifndef CHROMEOS_CAMERA_LIB_AUTO_FRAMING_AUTO_FRAMING_CROS_H_
#define CHROMEOS_CAMERA_LIB_AUTO_FRAMING_AUTO_FRAMING_CROS_H_

#include <stdint.h>

#include <memory>
#include <string>

namespace cros {

// Auto-framing pipeline interface for portable Chrome OS library.
class AutoFramingCrOS {
 public:
  // Image formats that map to mediapipe::ImageFormat::Format.
  enum class ImageFormat {
    kSRGB,
    kGRAY8,
  };

  // AutoFramingCrOS config options.
  struct Options {
    // Specify the input format that will be passed to ProcessFrame().
    ImageFormat input_format = ImageFormat::kGRAY8;
    int input_width = 0;
    int input_height = 0;
    double frame_rate = 0.0;

    // Aspect ratio of the output crop window.
    int target_aspect_ratio_x = 0;
    int target_aspect_ratio_y = 0;
  };

  // Callbacks for receiving auto-framing pipeline processed results.
  class Client {
   public:
    virtual ~Client() = default;

    // Called when a region of interest (ROI) is calculated.  ROI contains the
    // detected saliency points of the single frame without motion model
    // applied. Depending on the detection speed, this is not called on every
    // frame that passed with `ProcessFrame()`.
    virtual void OnNewRegionOfInterest(int64_t timestamp, int x_min, int y_min,
                                       int x_max, int y_max) = 0;
    // Called when a crop rectangle is calculcated.  Motion model is applied so
    // it should move smoothly.
    virtual void OnNewCropWindow(int64_t timestamp, int x_min, int y_min,
                                 int x_max, int y_max) = 0;
    // Called when an annotated image that visualizes the algorithm is rendered.
    virtual void OnNewAnnotatedFrame(int64_t timestamp, const uint8_t* data,
                                     int stride) = 0;
  };

  // Factory method to create an AutoFramingCrOS instance.
  static std::unique_ptr<AutoFramingCrOS> Create();

  virtual ~AutoFramingCrOS() = default;

  // Initialize with `options` and register callbacks with `client`.  Override
  // the built-in graph config with `graph_config_override` if not null.
  virtual bool Initialize(const Options& options, Client* client,
                          const std::string* graph_config_override) = 0;

  // Queue an input image for processing.  `data` contains image pixels in the
  // format specified with `Options::input_format` and `stride`.  `timestamp` is
  // carried to the Client callbacks.  `data` should live long enough until
  // processed.
  virtual bool ProcessFrame(int64_t timestamp, const uint8_t* data,
                            int stride) = 0;

  // Wait until all the queued frames are processed.
  virtual bool Wait() = 0;
};

}  // namespace cros

#endif  // CHROMEOS_CAMERA_LIB_AUTO_FRAMING_AUTO_FRAMING_CROS_H_
